import requests
from requests import Response

BASE = "https://qiita.com/api/v2/"


class Route:
    def __init__(
            self,
            method: str,
            path: str,
            token: str,
            *args,
            **kwargs
    ):
        __method_type__ = {
            "GET",
            "POST",
            "DELETE",
            "PUT",
            "PATCH"
        }
        if method not in __method_type__:
            raise TypeError(f"This method not in {__method_type__}")

        self.method = method
        self.path = path
        self.token = token

        self.url = BASE + self.path

        self.headers = {
            "Authorization": f"Bearer {self.token}"
            # "Content-Type": "application/json"
        }

        data = requests.request(
            method=method,
            url=self.url,
            headers=self.headers,
            json=kwargs.get("json")
        )
        self.data = data

    @property
    def base(self) -> str:
        return BASE

    @property
    def response(self) -> Response:
        return self.data




from datetime import datetime
from typing import Dict, Any

from .team import Team


class Group:
    def __init__(self, data: Dict[str, Any]):
        self.data = data
        # self.__created_at: str = data.get("created_at")
        # self.created_at: datetime = datetime.strptime(self.__created_at, "%Y-%m-%dT%H:%M:%S%z")
        # self.description: str = data.get("description")
        # self.name: str = data.get("name")
        # self.private: bool = True if str(data.get("private")) == "True" else False
        # self.__updated_at: str = data.get("updated_at")
        # self.updated_at: datetime = datetime.strptime(self.__updated_at, "%Y-%m-%dT%H:%M:%S%z")
        # self.url_name: str = data.get("url_name")

    def __repr__(self) -> str:
        return (
            f"<Group group_name={self.name} description={self.description}>"
        )

    @property
    def created_at(self) -> datetime:
        _created_at: str = self.data.get("created_at")
        return datetime.strptime(_created_at, "%Y-%m-%dT%H:%M:%S%z")

    @property
    def description(self) -> str:
        return self.data.get("description")

    @property
    def name(self) -> str:
        return self.data.get("name")

    @property
    def private(self) -> bool:
        return True if self.data.get("private").lower() == "True" else False

    @property
    def updated_at(self) -> datetime:
        _updated_at = self.data.get("updated_at")
        return datetime.strptime(_updated_at, "%Y-%m-%dT%H:%M:%S%z")

    @property
    def url_name(self) -> str:
        return self.data.get("url_name")



from datetime import datetime
from typing import Dict, Any

from .member import Member


class TeamMember(Member):
    def __init__(self, data: Dict[str, Any]):
        super().__init__(data)
        # self.description: str = data.get("description")
        # self.__last_accessed_at: str = data.get("last_accessed_at")
        # self.last_accessed_at: datetime = datetime.strptime(self.__last_accessed_at, "%Y-%m-%dT%H:%M:%S%z")

    @property
    def description(self) -> str:
        return self.data.get("description")

    @property
    def last_accessed_at(self) -> datetime:
        _last: str = self.data.get("last_accessed_at")
        return datetime.strptime(_last, "%Y-%m-%dT%H:%M:%S%z")



from datetime import datetime
from typing import Dict, Any, Optional


class Project:
    def __init__(self, data: Dict[str, Any]):
        self.data = data
        # self.rendered_body: str = data.get("rendered_body")
        # self.archived: bool = True if str(data.get("archived")) == "True" else False
        # self.body = data.get("body")
        # self.__created_at: str = data.get("created_at")
        # self.created_at: datetime = datetime.strptime(self.__created_at, "%Y-%m-%dT%H:%M:%S%z")
        # self.id = data.get("id")
        # self.name = data.get("name")
        # self.reactions_count = data.get("reactions_count")
        # self.__updated_at: str = data.get("updated_at")
        # self.updated_at: datetime = datetime.strptime(self.__updated_at, "%Y-%m-%dT%H:%M:%S%z")

    @property
    def rendered_body(self) -> str:
        return self.data.get("rendered_body")

    @property
    def is_archived(self) -> bool:
        return True if self.data.get("archived").lower() == "true" else False

    @property
    def body(self) -> str:
        return self.data.get("body")

    @property
    def created_at(self) -> datetime:
        _created_at = self.data.get("created_at")
        return datetime.strptime(_created_at, "%Y-%m-%dT%H:%M:%S%z")

    @property
    def id(self) -> str:
        return self.data.get("id")

    @property
    def name(self) -> str:
        return self.data.get("name")

    @property
    def reactions_count(self) -> Optional[int]:
        count: str = self.data.get("reactions_count")
        return int(count) if count.isdigit() else None

    @property
    def updated_at(self) -> datetime:
        _updated_at = self.data.get("updated_at")
        return datetime.strptime(_updated_at, "%Y-%m-%dT%H:%M:%S%z")





from enum import Enum
from typing import List

from . import Group, GroupMember, Comment, Tagging, Team, TeamMember, Template, Project
from .exception import InvalidItemsRequested
from .item import Item
from .route import Route
from .tag import Tag
from .user import User


class Sort(Enum):
    COUNT = "count"
    NAME = "name"


class Client:
    def __init__(
            self,
            token
    ):
        if not isinstance(token, str):
            raise TypeError(
                f"token must be str, not {token.__class__.__name__}"
            )
        self.token = token

    # グループ
    def get_groups(self, page: int = 1, items: int = 20) -> List[Group]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested(
                "Too many items were requested. You can request more than 1 and less than 100."
            )

        response = Route(
            method="GET",
            token=self.token,
            path=f"groups?page={page}&per_page={items}"
        )
        return [Group(group) for group in response.response.json()]

    def get_group(self, url_name: str) -> Group:
        response = Route(
            method="GET",
            token=self.token,
            path=f"groups/{url_name}"
        )
        return Group(response.response.json())

    def create_group(self) -> None:
        pass

    def update_group(self) -> None:
        pass

    def delete_group(self, url_name: str) -> None:
        response = Route(
            method="DELETE",
            token=self.token,
            path=f"groups/{url_name}"
        )
        if response.response.status_code == 204:
            ...

    # グループメンバー
    def get_members(
            self,
            url_name: str,
            page: int = 1,
            items: int = 20
    ) -> List[GroupMember]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested(
                "Too many items were requested. You can request more than 1 and less than 100."
            )
        response = Route(
            method="GET",
            token=self.token,
            path=f"groups/{url_name}/members?page={page}&per_page={items}"
        )
        return [GroupMember(member) for member in response.response.json()]

    def get_member(
            self,
            url_name: str,
            user_id: str
    ) -> GroupMember:
        response = Route(
            method="GET",
            token=self.token,
            path=f"groups/{url_name}/members/{user_id}"
        )
        return GroupMember(response.response.json())

    def add_member(
            self,
            url_name: str,
            user_id: str = None,
            email: str = None
    ):
        json = dict()
        if user_id:
            json["identities"] = [user_id]

        elif email:
            json["identities"] = [email]

        else:
            raise ValueError()

        response = Route(
            method="POST",
            token=self.token,
            path=f"groups/{url_name}/members",
            json=json
        )

    def delete_member(
            self,
            url_name: str,
            user_id: str = None,
            email: str = None
    ):
        json = dict()
        if user_id:
            json["identities"] = [user_id]

        elif email:
            json["identities"] = [email]

        else:
            raise ValueError()

        response = Route(
            method="DELETE",
            token=self.token,
            path=f"groups/{url_name}/members",
            json=json
        )

    # コメント
    def delete_comment(
            self,
            comment_id: str
    ) -> bool:
        response = Route(
            method="DELETE",
            token=self.token,
            path=f"comments/{comment_id}"
        )
        if response.response.status_code == 404:
            return True

        return False

    def get_comment(
            self,
            comment_id: str
    ) -> Comment:
        response = Route(
            method="GET",
            token=self.token,
            path=f"comments/{comment_id}"
        )
        if response.response.status_code == 200:
            return Comment(response.response.json())
        return None

    def get_comments(
            self,
            item_id: str
    ) -> List[Comment]:
        response = Route(
            method="GET",
            token=self.token,
            path=f"items/{item_id}/comments"
        )
        if response.response.status_code == 200:
            return [Comment(comment) for comment in response.response.json()]
        return None

    def post_comment(
            self,
            item_id: str,
            comment: str
    ) -> Comment:
        response = Route(
            method="POST",
            token=self.token,
            path=f"items/{item_id}/comments"
        )
        if response.response.status_code == 201:
            return Comment(response.response.json())
        return None

    def post_user_comment(self, item_id: str): ...

    def get_project_comments(
            self,
            project_id: str
    ) -> Comment:
        response = Route(
            method="GET",
            token=self.token,
            path=f"projects/{project_id}/comments"
        )
        if response.response.status_code == 200:
            return Comment(response.response.json())
        return None

    def post_project_comment(
            self,
            project_id: str,
            comment: str
    ) -> Comment:
        response = Route(
            method="POST",
            token=self.token,
            path=f"projects/{project_id}",
            json={"body": comment}
        )
        if response.response.status_code == 201:
            return Comment(response.response.json())
        return None

    def post_user_project_comment(
            self,
            project_id: str,
            comment: str,
            user_id: str
    ) -> Comment:
        response = Route(
            method="POST",
            token=self.token,
            path=f"projects/{project_id}/imported_comments",
            json={"body": comment, "user_id": user_id}
        )
        if response.response.status_code == 201:
            return Comment(response.response.json())
        return None

    def edit_comment(
            self,
            comment_id: str,
            comment: str
    ) -> Comment:
        response = Route(
            method="PATCH",
            token=self.token,
            path=f"comments/{comment_id}",
            json={"body": comment}
        )
        if response.response.status_code == 200:
            return Comment(response.response.json())
        return None

    # タギング
    def set_tag(
            self,
            item_id: str,
            tag_name: str,
            versions: List[str] = None
    ) -> Tagging:
        json = dict()
        json["name"] = tag_name
        if versions:
            json["versions"] = versions
        response = Route(
            method="POST",
            token=self.token,
            path=f"items/{item_id}/tagging",
            json=json
        )
        if response.response.status_code == 201:
            return Tagging(response.response.json())
        return None

    def delete_tag(
            self,
            item_id: str,
            tagging_id: str
    ):
        response = Route(
            method="DELETE",
            token=self.token,
            path=f"items/{item_id}/tagging/{tagging_id}"
        )
        if response.response.status_code == 204:
            return True
        return False

    # タグ
    def get_tag(
            self,
            tag_id: str
    ) -> Tag:
        response = Route(
            method="GET",
            token=self.token,
            path=f"tags/{tag_id}"
        )
        if response.response.status_code == 200:
            return Tag(response.response.json())
        return None

    def get_tags(self, sort: Sort, page: int = 1, items: int = 20) -> List[Tag]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested("Too many items were requested. You can request more than 1 and less than 100.")

        response = Route(
            method="GET",
            token=self.token,
            path=f"tags?page={page}&per_page={items}"
        )
        return [Tag(data) for data in response.response.json()]

    def get_following_tags(
            self,
            user_id: str,
            page: int = 1,
            items: int = 20
    ) -> List[Tag]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested(
                "Too many items were requested. You can request more than 1 and less than 100."
            )
        response = Route(
            method="GET",
            token=self.token,
            path=f"users/{user_id}/following_tags?page={page}&per_page={items}"
        )
        if response.response.status_code == 200:
            return [Tag(tag) for tag in response.response.json()]
        return None

    def delete_following_tag(
            self,
            tag_id: str
    ):
        response = Route(
            method="DELETE",
            token=self.token,
            path=f"tags/{tag_id}/following"
        )
        if response.response.status_code == 200:
            return True
        return False

    def follow_tag(
            self,
            tag_id: str
    ):
        response = Route(
            method="PUT",
            token=self.token,
            path=f"tags/{tag_id}/following"
        )
        if response.response.status_code == 204:
            return True
        return False

    # チーム
    def get_user_teams(self) -> List[Team]:
        response = Route(
            method="GET",
            token=self.token,
            path=f"teams"
        )
        if response.response.status_code == 200:
            return [Team(team) for team in response.response.json()]
        return None

    # チームメンバー
    def get_team_members(
            self,
            items: int = 1,
            page: int = 20
    ) -> List[TeamMember]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested(
                "Too many items were requested. You can request more than 1 and less than 100."
            )
        response = Route(
            method="GET",
            token=self.token,
            path=f"team_memberships?page={items}&per_page={page}"
        )
        if response.response.status_code == 200:
            return [TeamMember(member) for member in response.response.json()]
        return None

    # テンプレート
    def get_templates(
            self,
            page: int = 1,
            items: int = 20
    ) -> List[Template]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested(
                "Too many items were requested. You can request more than 1 and less than 100."
            )
        response = Route(
            method="GET",
            token=self.token,
            path=f"templates?page={page}&per_page={items}"
        )
        if response.response.status_code == 200:
            return [Template(temp) for temp in response.response.json()]
        return None

    def delete_template(
            self,
            template_id: str
    ):
        response = Route(
            method="DELETE",
            token=self.token,
            path=f"templates/{template_id}"
        )
        if response.response.status_code == 200:
            return True
        return False

    def get_template(
            self,
            template_id: str
    ) -> Template:
        response = Route(
            method="GET",
            token=self.token,
            path=f"templates/{template_id}"
        )
        if response.response.status_code == 200:
            return Template(response.response.json())
        return None

    def add_template(self): ...

    def update_template(self): ...

    def get_projects(
            self,
            page: int = 1,
            items: int = 20
    ) -> List[Project]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested(
                "Too many items were requested. You can request more than 1 and less than 100."
            )
        response = Route(
            method="GET",
            token=self.token,
            path=f"projects?page={page}&per_page={items}"
        )
        if response.response.status_code == 200:
            return [Project(project) for project in response.response.json()]
        return None

    def add_project(self): ...

    def delete_project(self): ...

    def get_project(
            self,
            project_id: str
    ):
        response = Route(
            method="GET",
            token=self.token,
            path=f"projects/{project_id}"
        )
        if response.response.status_code == 200:
            return Project(response.response.json())
        return None

    def update_project(self): ...

    # ユーザー
    def get_authenticated_user_data(self) -> User:
        response = Route(
            method="GET",
            token=self.token,
            path="authenticated_user"
        )
        return User(response.response.json())

    def get_stockers(
            self,
            item_id: str,
            page: int = 1,
            items: int = 20
    ) -> List[Item]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested(
                "Too many items were requested. You can request more than 1 and less than 100."
            )

        response = Route(
            method="GET",
            token=self.token,
            path=f"items/{item_id}/stockers?page={page}&per_page={items}"
        )
        if response.response.status_code == 200:
            return [User(item) for item in response.response.json()]
        return None

    def get_user(
            self,
            user_id: str
    ) -> User:
        response = Route(
            method="GET",
            token=self.token,
            path=f"users/{user_id}"
        )
        if response.response.status_code == 200:
            return User(response.response.json())
        return None

    def get_users(
            self,
            page: int = 1,
            items: int = 20
    ) -> List[User]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested(
                "Too many items were requested. You can request more than 1 and less than 100."
            )

        response = Route(
            method="",
            token=self.token,
            path=f"users?page={page}&per_page={items}"
        )
        if response.response.status_code == 200:
            return [User(json) for json in response.response.json()]
        return None

    def get_followees(
            self,
            user_id: str,
            page: int = 1,
            items: int = 20
    ) -> List[User]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested(
                "Too many items were requested. You can request more than 1 and less than 100."
            )

        response = Route(
            method="GET",
            token=self.token,
            path=f"users/{user_id}/followees?page={page}&per_page={items}"
        )
        if response.response.status_code == 200:
            return [User(json) for json in response.response.json()]
        return None

    def get_followers(
            self,
            user_id: str,
            page: int = 1,
            items: int = 20
    ) -> List[User]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested(
                "Too many items were requested. You can request more than 1 and less than 100."
            )

        response = Route(
            method="GET",
            token=self.token,
            path=f"users/{user_id}/followers?page={page}&per_page={items}"
        )
        if response.response.status_code == 200:
            return [User(json) for json in response.response.json()]
        return None

    def delete_following(
            self,
            user_id: str
    ) -> bool:
        response = Route(
            method="DELETE",
            token=self.token,
            path=f"users/{user_id}/following"
        )
        if response.response.status_code == 204:
            return True
        return False

    def is_following(
            self,
            user_id: str
    ) -> bool:
        response = Route(
            method="GET",
            token=self.token,
            path=f"users/{user_id}/following"
        )
        if response.response.status_code == 204:
            return True
        return False

    def set_following(
            self,
            user_id: str
    ) -> bool:
        response = Route(
            method="PUT",
            token=self.token,
            path=f"users/{user_id}/following"
        )
        if response.response.status_code == 204:
            return True
        return False

    # 投稿
    def get_item(
            self,
            item_id: str
    ) -> Item:
        response = Route(
            "GET",
            token=self.token,
            path=f"items/{item_id}"
        )
        if response.response.status_code == 200:
            return Item(response.response.json())
        return None

    def get_items(
            self,
            page: int = 1,
            items: int = 20
    ) -> List[Item]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested(
                "Too many items were requested. You can request more than 1 and less than 100."
            )

        response = Route(
            method="GET",
            token=self.token,
            path=f"authenticated_user/items?page={page}&per_page={items}"
        )
        if response.response.status_code == 200:
            return [Item(json) for json in response.response.json()]
        return None

    def delete_item(
            self,
            item_id: str
    ) -> bool:
        response = Route(
            method="DELETE",
            token=self.token,
            path=f"items/{item_id}"
        )
        if response.response.status_code == 200:
            return True
        return False

    def is_stock(
            self,
            item_id: str
    ) -> bool:
        response = Route(
            method="GET",
            token=self.token,
            path=f"items/{item_id}/stock"
        )
        if response.response.status_code == 204:
            return True
        return False

    def set_stock(
            self,
            itme_id: str
    ) -> bool:
        response = Route(
            method="PUT",
            token=self.token,
            path=f"items/{itme_id}/stock"
        )
        if response.response.status_code == 204:
            return True
        return False

    def get_tagging_items(
            self,
            tag_id: str,
            page: int = 1,
            items: int = 20
    ) -> List[Item]:
        if items > 100 | items <= 0:
            raise InvalidItemsRequested("Too many items were requested. You can request more than 1 and less than 100.")
        response = Route(
            method="GET",
            token=self.token,
            path=f"tags/{tag_id}/items?page={page}&per_page={items}"
        )
        if response.response.status_code == 200:
            return [Item(item) for item in response.response.json()]
        return None




from typing import Dict, Any, List, Optional, TYPE_CHECKING


class Tagging:
    def __init__(self, data: Dict[str, Any]):
        self.data = data
        # self.name: str = data.get("name")
        # self.versions: List[Optional[str]] = data.get("versions")

    def __repr__(self) -> str:
        return (
            f"<Tagging name={self.name}>"
        )

    @property
    def name(self) -> str:
        return self.data.get("name")

    @property
    def versions(self) -> List[str]:
        return self.data.get("versions")



from typing import Dict, Any


class Member:
    def __init__(self, data: Dict[str, Any]):
        self.data = data
        # self.id: str = data.get("id")
        # self.name: str = data.get("name")
        # self.email: str = data.get("email")

    @property
    def id(self) -> str:
        return self.data.get("id")

    @property
    def name(self) -> str:
        return self.data.get("name")

    @property
    def email(self) -> str:
        return self.data.get("email")



from typing import Optional, Dict, Any


class User:
    def __init__(self, data: Dict[str, Any]):
        self.data: Dict[str, Any] = data

    def __repr__(self) -> str:
        return (
            f"<User user_id={self.id} followees_count={self.followees_count}"
            f" followers_count={self.followers_count} items_count={self.items_count}>"
        )

    @property
    def description(self) -> Optional[str]:
        return self.data.get("description")

    @property
    def facebook_id(self) -> Optional[str]:
        return self.data.get("facebook_id")

    @property
    def followees_count(self) -> int:
        return int(self.data.get("followees_count"))

    @property
    def followers_count(self) -> int:
        return int(self.data.get("followers_count"))

    @property
    def github_login_name(self) -> Optional[str]:
        return self.data.get("github_login_name")

    @property
    def id(self) -> str:
        return self.data.get("id")

    @property
    def items_count(self) -> int:
        return int(self.data.get("items_count"))

    @property
    def linkedin_id(self) -> Optional[str]:
        return self.data.get("linkedin_id")

    @property
    def location(self) -> Optional[str]:
        return self.data.get("location")

    @property
    def name(self) -> Optional[str]:
        return self.data.get("name")

    @property
    def organization(self) -> Optional[str]:
        return self.data.get("organization")

    @property
    def permanent_id(self) -> str:
        return self.data.get("permanent_id")

    @property
    def profile_image_url(self) -> str:
        return self.data.get("profile_image_url")

    @property
    def team_only(self) -> bool:
        return True if self.data.get("team_only").lower() == "true" else False

    @property
    def twitter_screen_name(self) -> Optional[str]:
        return self.data.get("twitter_screen_name")

    @property
    def website_url(self) -> Optional[str]:
        return self.data.get("website_url")

    @property
    def image_monthly_upload_limit(self) -> Optional[int]:
        limit = self.data.get("image_monthly_upload_limit")
        return int(limit) if limit is not None else None

    @property
    def image_monthly_upload_remaining(self) -> Optional[int]:
        remain = self.data.get("image_monthly_upload_remaining")
        return int(remain) if remain is not None else None



from datetime import datetime
from typing import Dict, Any, Optional

from .member import Member
from .tagging import Tagging
from .team import Team
from .user import User


class Item:
    def __init__(self, data: Dict[str, Any]):
        self.data = data
        # self.rendered_body: str = data.get("rendered_body")
        # self.body: str = data.get("body")
        # self.coediting: bool = data.get("coediting")
        # self.comments_count: int = int(data.get("comments_count"))
        # self.__created_at: str = data.get("created_at")
        # self.created_at: datetime = datetime.strptime(self.__created_at, "%Y-%m-%dT%H:%M:%S%z")
        # self.group: Team = data.get("group")
        # self.id: str = data.get("id")
        # self.likes_count: int = int(data.get("likes_count"))
        # self.private: bool = True if str(data.get("private")) == "True" else False
        # self.reactions_count: int = int(data.get("reactions_count"))
        # self.tags: List[Dict[str, Union[str, List[str]]]] = data.get("tags")
        # self.title: str = data.get("title")
        # self.__updated_at: str = data.get("updated_at")
        # self.updated_at: datetime = datetime.strptime(self.__updated_at, "%Y-%m-%dT%H:%M:%S%z")
        # self.url: str = data.get("url")
        # self.user: User = User(data.get("user"))
        # self.page_views_count: Optional[int] = int(data.get("page_views_count"))
        # self.team_membership: Member = data.get("team_membership")

    def __repr__(self) -> str:
        return (
            f"<Item user_id={self.user.id} article_id={self.id}"
            f" views_count={self.page_views_count} url={self.url}>"
        )

    @property
    def rendered_body(self) -> str:
        return self.data.get("rendered_body")

    @property
    def body(self) -> str:
        return self.data.get("body")

    @property
    def coediting(self) -> bool:
        if self.data.get("coediting").lower() == "true":
            return True
        return False

    @property
    def comments_count(self) -> int:
        return int(self.data.get("comments_count"))

    @property
    def created_at(self) -> datetime:
        _created_at: str = self.data.get("created_at")
        return datetime.strptime(_created_at, "%Y-%m-%dT%H:%M:%S%z")

    @property
    def group(self) -> Team:
        return Team(self.data.get("group"))

    @property
    def id(self) -> str:
        return self.data.get("id")

    @property
    def likes_count(self) -> int:
        return int(self.data.get("likes_count"))

    @property
    def private(self) -> bool:
        if self.data.get("private") == "True":
            return True
        return False

    @property
    def reactions_count(self) -> int:
        return int(self.data.get("reactions_count"))

    @property
    def tags(self) -> Tagging:
        return Tagging(self.data.get("tags"))

    @property
    def title(self) -> str:
        return self.data.get("title")

    @property
    def updated_at(self) -> datetime:
        _updated_at = self.data.get("updated_at")
        return datetime.strptime(_updated_at, "%Y-%m-%dT%H:%M:%S%z")

    @property
    def url(self) -> str:
        return self.data.get("url")

    @property
    def user(self) -> User:
        return User(self.data.get("user"))

    @property
    def page_views_count(self) -> Optional[int]:
        count: str = self.data.get("page_views_count")
        return int(count) if count else None

    @property
    def team_membership(self) -> Member:
        return Member(self.data.get("team_membership"))




import json

import requests

BASE = "https://qiita.com/api/v2"

token = "6ed96829083437217f9c077c61d225fcb619d7ca"
Auth = {"Authorization": f"Bearer {token}"}

url = BASE + "/authenticated_user"

data = dict()

data["Auth"] = f"Bearer {token}"

r = requests.get(url=url, headers=Auth)
print(r)
print(r.json())



from datetime import datetime
from typing import Dict, Any

from .user import User


class Reaction:
    def __init__(self, data: Dict[str, Any]):
        self.data = data
        # self.__created_at: str = data.get("created_at")
        # self.created_at: datetime = datetime.strptime(self.__created_at, "%Y-%m-%dT%H:%M:%S%z")
        # self.image_url: str = data.get("image_url")
        # self.name: str = data.get("name")
        # self.user: User = data.get("user")

    @property
    def created_at(self) -> datetime:
        _created_at: str =  self.data.get("created_at")
        return datetime.strptime(_created_at, "%Y-%m-%dT%H:%M:%S%z")

    @property
    def image_url(self) -> str:
        return self.data.get("image_url")

    @property
    def name(self) -> str:
        return self.data.get("name")

    @property
    def user(self) -> User:
        return User(self.data.get("user"))



from datetime import datetime
from typing import Any, Dict

from .user import User


class Comment:
    def __init__(self, data: Dict[str, Any]):
        self.data = data

    @property
    def body(self) -> str:
        return self.data.get("body")

    @property
    def created_at(self) -> datetime:
        _created_at: str = self.data.get("created_at")
        return datetime.strptime(_created_at, "%Y-%m-%dT%H:%M:%S%z")

    @property
    def id(self) -> str:
        return self.data.get("id")

    @property
    def rendered_body(self) -> str:
        return self.data.get("rendered_body")

    @property
    def updated_at(self) -> datetime:
        _updated_at = self.data.get("updated_at")
        return datetime.strptime(_updated_at, "%Y-%m-%dT%H:%M:%S%z")

    @property
    def user(self) -> User:
        return User(self.data.get("user"))



from setuptools import setup
import re

version = ""

with open("Qiipy/__init__.py", mode="r", encoding="utf8") as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

requirement = "requests"

setup(
    name='pyqiita',
    version=version,
    packages=['qiipy'],
    url='https://github.com/peco2282/Qiipy',
    license='MIT',
    author='peco2282',
    author_email='pecop2282@gmail.com',
    description='An API wrapper for Qiita.'
)



from typing import Dict, Any


class Tag:
    def __init__(self, data: Dict[str, Any]):
        self.data = data
        # self.followers_count = data.get("followers_count")
        # self.icon_url = data.get("icon_url")
        # self.id = data.get("id")
        # self.items_count = data.get("items_count")

    def __repr__(self):
        return (
            f"<Tag id={self.id} followers={self.followers_count} items={self.items_count}>"
        )

    @property
    def followers_count(self) -> int:
        return int(self.data.get("followers_count"))

    @property
    def icon_url(self) -> str:
        return self.data.get("icon_url")

    @property
    def id(self) -> str:
        return self.data.get("id")

    @property
    def items_count(self) -> int:
        return int(self.data.get("items_count"))



from typing import Any, Dict, List, Union

from .tagging import Tagging
from .group import Group


class Template:
    def __init__(self, data: Dict[str, Any]):
        self.data = data
        # self.body: str = data.get("body")
        # self.id: int = int(data.get("id"))
        # self.name: str = data.get("name")
        # self.expanded_body: str = data.get("expanded_body")
        # self.expanded_tags: List[Dict[str, Union[str, List[str]]]] = data.get("expanded_tags")
        # self.expanded_title: str = data.get("expanded_title")
        # self.group: Group = data.get("group")
        # self.tags: List[Dict[str, Union[str, List[str]]]] = data.get("tags")
        # self.title: str = data.get("title")
        # self.coedit: bool = True if str(data.get("coedit")) == "True" else False

    @property
    def body(self) -> str:
        return self.data.get("body")

    @property
    def id(self) -> int:
        _id: str = self.data.get("id")
        return int(_id) if _id.isdigit() else None

    @property
    def name(self) -> str:
        return self.data.get("name")

    @property
    def expanded_body(self) -> str:
        return self.data.get("expanded_body")

    @property
    def expanded_tags(self) -> Tagging:
        return Tagging(self.data.get("expanded_tags"))

    @property
    def expanded_title(self) -> str:
        return self.data.get("expanded_title")

    @property
    def group(self) -> Group:
        return Group(self.data.get("group"))

    @property
    def tags(self) -> Tagging:
        return Tagging(self.data.get("tags"))

    @property
    def title(self) -> str:
        return self.data.get("title")

    @property
    def coedit(self) -> bool:
        return True if self.data.get("coedit").lower() == "true" else False



from typing import Dict, Any


class Team:
    def __init__(self, data: Dict[str, Any]):
        self.data = data
        # self.is_active: bool = True if str(data.get("active")) == "True" else False
        # self.id: str = data.get("id")
        # self.name: str = data.get("name")

    def __repr__(self):
        return (
            f"<Team name={self.name} id={self.id} active={self.is_active}>"
        )

    @property
    def is_active(self) -> bool:
        return True if self.data.get("is_active").lower() == "true" else False

    @property
    def id(self) -> str:
        return self.data.get("id")

    @property
    def name(self) -> str:
        return self.data.get("name")




__title__ = "qiipy"
__anthor__ = "peco2282"
__license__ = "MIT"
__version__ = "1.0.0"

from .client import *
from .comment import *
from .exception import *
from .group import *
from .group_member import *
from .item import *
from .member import *
from .project import *
from .reaction import *
from .route import *
from .tag import *
from .tagging import *
from .team import *
from .team_member import *
from .template import *
